package SmartKarrot;

import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.CallbackContext;

import org.json.JSONArray;
import org.json.JSONException;

import android.content.Context;
import android.support.annotation.NonNull;

import com.google.firebase.iid.FirebaseInstanceId;
//import com.google.firebase.iid.InstanceIdResult;

import com.google.firebase.messaging.RemoteMessage;
import com.smartkarrot.survey.Survey;
import com.smartkarrot.usageanalytics.EventParameters;
import com.smartkarrot.usageanalytics.RichPush;
import com.smartkarrot.usageanalytics.UsageAnalytics;
import com.smartkarrot.usageanalytics.UserAttribute;
import com.smartkarrot.referral.Referral;
import com.smartkarrot.usageanalytics.FirebaseTokenFactory;

import org.json.JSONObject;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

public class SmartKarrot extends CordovaPlugin {

  @Override
  public boolean execute(String action, JSONArray args, CallbackContext callbackContext) throws JSONException {
    if (action.equals("configure")) {
      String appId = args.getString(0);
      String orgId = args.getString(1);
      this.configure(appId, orgId, callbackContext);
      return true;
    } else if (action.equals("setUserId")) {
      String userId = args.getString(0);
      this.setUserId(userId, callbackContext);
      return true;
    } else if (action.equals("resetUserId")) {
      this.resetUserId(callbackContext);
      return true;
    } else if (action.equals("setUserAttribute")) {
      String key = args.getString(0);
      String value = args.getString(1);
      this.setUserAttribute(key, value, callbackContext);
      return true;
    } else if (action.equals("logEvent")) {
      String event = args.getString(0);
      String withData = args.getString(1);
      this.logEvent(event, withData, callbackContext);
      return true;
    } else if (action.equals("configureSurvey")) {
      this.configureSurvey(callbackContext);
      return true;
    } else if (action.equals("listSurveys")) {
      this.listSurveys(callbackContext);
      return true;
    } else if (action.equals("startSurvey")) {
      String surveyId = args.getString(0);
      this.startSurvey(surveyId, callbackContext);
      return true;
    } else if (action.equals("handleRichPush")) {
      RemoteMessage message = (RemoteMessage) args.get(0);
      this.handleRichPush(message);
      return true;
    } else if (action.equals("configureReferral")) {
      this.configureReferral(callbackContext);
      return true;
    } else if (action.equals("listReferrals")) {
      this.listReferrals(callbackContext);
      return true;
    } else if (action.equals("showDashboard")) {
      String referralId = args.getString(0);
      this.showDashboard(referralId, callbackContext);
    }
    return false;
  }

  private void configure(String appId, String orgId, CallbackContext callbackContext) {
    Context context = this.cordova.getActivity().getApplicationContext();

    /*UsageAnalytics.getInstance().setTokenFactory(new FirebaseTokenFactory() {
      @Override
      public void updateToken() {
        FirebaseInstanceId.getInstance().getInstanceId().addOnCompleteListener(new OnCompleteListener<InstanceIdResult>() {
          @Override
          public void onComplete(@NonNull Task<InstanceIdResult> task) {
            if (task.isSuccessful())
              UsageAnalytics.updateDeviceToken(task.getResult().getToken());
          }
        });
      }
    });*/

    if (appId != null && orgId != null) {
      UsageAnalytics.init(context, appId, orgId);
      callbackContext.success("SmartKarrot stage configuration Successful");
    } else if (appId != null) {
      UsageAnalytics.init(context, appId);
      callbackContext.success("SmartKarrot production configuration Successful");
    } else {
      callbackContext.error("App Id is required.");
    }
  }

  private void setUserId(String userId, CallbackContext callbackContext) {
    if (userId != null) {
      UsageAnalytics.getInstance().setUserId(userId);
      callbackContext.success("User Id set Successfully");
    } else {
      callbackContext.error("User Id set UnSuccessfull");
    }
  }

  private void resetUserId(CallbackContext callbackContext) {
    UsageAnalytics.getInstance().resetUserId();
    callbackContext.success("User Id reset Successfull");
  }

  private void setUserAttribute(String key, String value, CallbackContext callbackContext) {
    if (key != null) {
      if (key == UserAttribute.country.name()) {
        UsageAnalytics.getInstance().setUserAttribute(UserAttribute.country, value);
      } else if (key == UserAttribute.emailId.name()) {
        UsageAnalytics.getInstance().setUserAttribute(UserAttribute.emailId, value);
      } else if (key == UserAttribute.mobileNumber.name()) {
        UsageAnalytics.getInstance().setUserAttribute(UserAttribute.mobileNumber, value);
      } else if (key == UserAttribute.dateOfBirth.name()) {
        UsageAnalytics.getInstance().setUserAttribute(UserAttribute.dateOfBirth, value);
      } else if (key == UserAttribute.gender.name()) {
        UsageAnalytics.getInstance().setUserAttribute(UserAttribute.gender, value);
      } else if (key == UserAttribute.name.name()) {
        UsageAnalytics.getInstance().setUserAttribute(UserAttribute.name, value);
      } else if (key == UserAttribute.language.name()) {
        UsageAnalytics.getInstance().setUserAttribute(UserAttribute.language, value);
      } else if (key == UserAttribute.userId.name()) {
        UsageAnalytics.getInstance().setUserAttribute(UserAttribute.userId, value);
      } else {
        UsageAnalytics.getInstance().setUserAttribute(key, value);
      }
      callbackContext.success("User attribute set Successfull");

    } else {
      callbackContext.error("User attribute set UnSuccessfull");
    }
  }

  private static Map<String, String> parseJSONObjectToMap(JSONObject jsonObject) throws JSONException {
    Map<String, String> mapData = new HashMap<String, String>();
    Iterator<String> keysItr = jsonObject.keys();
    while (keysItr.hasNext()) {
      String key = keysItr.next();
      String value = jsonObject.getString(key);

      mapData.put(key, value);
    }
    return mapData;
  }

  private void logEvent(String event, String withData, CallbackContext callbackContext) {
    if (event != null) {
      if (withData != null && withData != "null") {
        JSONObject jo = null;
        try {
          jo = new JSONObject(withData);
          Map<String, String> mappedString = parseJSONObjectToMap(jo);
          int index = 0;
          EventParameters.Builder b = null;
          for (Map.Entry<String, String> entry : mappedString.entrySet()) {

            if (index == 0) {
              b = new EventParameters.Builder().addParameter(entry.getKey(), entry.getValue());
              index++;
            } else {
              b.addParameter(entry.getKey(), entry.getValue());
            }
          }
          EventParameters eventParameters = b.build();
          UsageAnalytics.getInstance().logEvent("", eventParameters);
        } catch (JSONException e) {

        }
      } else {
        UsageAnalytics.getInstance().logEvent(event);
      }
      callbackContext.success("Event Logged Successfull");
    } else {
      callbackContext.error("Event Logged UnSuccessfull");
    }
  }

  private void configureSurvey(CallbackContext callbackContext) {
    Context context = this.cordova.getActivity().getApplicationContext();
    Survey.init(context);
    callbackContext.success("SmartKarrot Survey configuration Successful");
  }

  private void listSurveys(CallbackContext callbackContext) {
    Context context = this.cordova.getActivity().getApplicationContext();
    Survey.getInstance().listSurveys(context);
    callbackContext.success("SmartKarrot Survey List configuration Successful");
  }

  private void startSurvey(String surveyId, CallbackContext callbackContext) {
    if (surveyId != null) {
      Context context = this.cordova.getActivity().getApplicationContext();
      Survey.getInstance().startSurvey(context, surveyId);
      callbackContext.success("SmartKarrot Survey By Id configuration Successful");
    } else {
      callbackContext.error("user Id & survey Id is required.");
    }
  }

  private void updateDeviceToken(String deviceToken, CallbackContext callbackContext) {
    if (deviceToken != null) {
      UsageAnalytics.updateDeviceToken(deviceToken);
      callbackContext.success("SmartKarrot Device Token Update Successful");
    } else {
      callbackContext.error("SmartKarrot Device Token Update UnSuccessful");
    }
  }

  private void configureReferral(CallbackContext callbackContext) {
    //Referral.configure();
    Context context = this.cordova.getActivity().getApplicationContext();
    Referral.init(context);
    callbackContext.success("SmartKarrot Configure Referral Successful");
  }

  private void listReferrals(CallbackContext callbackContext) {
    Context context = this.cordova.getActivity().getApplicationContext();
    Referral.getInstance().listCampaigns(context);
    callbackContext.success("SmartKarrot referral list Successful");
  }

  private void showDashboard(String referralId, CallbackContext callbackContext) {
    Context context = this.cordova.getActivity().getApplicationContext();
    Referral.getInstance().showCampaignDashboard(context, referralId);
    callbackContext.success("SmartKarrot show dashboard Successful");
  }

  private void handleRichPush(RemoteMessage remoteMessage) {
    Context context = this.cordova.getActivity().getApplicationContext();
    boolean isRichPushNotification = RichPush.isRichPushNotification(remoteMessage.getData());
    if (isRichPushNotification) {
      // Push notification is SK RichPush.
      RichPush.handleRichPush(context, remoteMessage.getData());
      return;
    }
  }
}
