import UIKit
import UsageAnalytics
import Survey
import Referral

@objc(SmartKarrot) class SmartKarrot: CDVPlugin {

    @objc(configure:)
    func configure(_ command: CDVInvokedUrlCommand?) {
        
        var pluginResult = CDVPluginResult(
            status: CDVCommandStatus_ERROR,
            messageAs: "AppId is required for the SmartKarrot Integration."
        )
        let appId:String?
        appId = command!.arguments[0] as? String;
        let orgId = command!.arguments[1] as? String;
        let userId = command!.arguments[2] as? String ?? nil;
        
        
        if(appId != nil && orgId != nil){
            (userId == nil || userId?.isEmpty == true) ?
                UsageAnalytics.shared.configure(appId: appId!,organizationId: orgId!) :
                UsageAnalytics.shared.configure( appId: appId!, organizationId:orgId!, userId: userId)
            pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "SmartKarrot Configuration Successful")
        }
        else if(appId != nil){
            (userId == nil || userId?.isEmpty == true) ?
                UsageAnalytics.shared.configure(appId: appId!) :
                UsageAnalytics.shared.configure(appId: appId!, userId: userId)
            pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "SmartKarrot Configuration Successful")
        }
        
        self.commandDelegate!.send(
            pluginResult,
            callbackId: command!.callbackId
        )
    }
    
    @objc(setUserId:)
    func setUserId(_ command: CDVInvokedUrlCommand?) {
        var pluginResult = CDVPluginResult(
            status: CDVCommandStatus_ERROR,
            messageAs: "UserId is required for the SmartKarrot Integration."
        )
        let userId:String?
        userId = command!.arguments[0] as? String;
        
        if(userId != nil){
            UsageAnalytics.shared.set(userId: userId!)
            
            pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "User Id set Successful")
        }
        self.commandDelegate!.send(
            pluginResult,
            callbackId: command!.callbackId
        )
    }
    
    @objc(resetUserId:)
    func resetUserId(_ command: CDVInvokedUrlCommand?) {
        var pluginResult = CDVPluginResult(
            status: CDVCommandStatus_ERROR,
            messageAs: "UserId is required for the SmartKarrot Integration."
        )
        
        UsageAnalytics.shared.resetUserId()
        
        
        pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "User Id reset Successful")
        
        self.commandDelegate!.send(
            pluginResult,
            callbackId: command!.callbackId
        )
    }
    
    @objc(setUserAttribute:)
    func setUserAttribute(_ command: CDVInvokedUrlCommand?) {
        var pluginResult = CDVPluginResult(
            status: CDVCommandStatus_ERROR,
            messageAs: "Error while setting User Attribute"
        )
        
        let key:String?
        key = command!.arguments[0] as? String;
        let value = command!.arguments[1] as? String ?? nil;
        
        switch key {
        case UserAttribute.country.rawValue:
            UsageAnalytics.shared.set(value: value!, for: .country)
        case UserAttribute.dateOfBirth.rawValue:
            UsageAnalytics.shared.set(value: value!, for: .dateOfBirth)
        case UserAttribute.emailId.rawValue:
            UsageAnalytics.shared.set(value: value!, for: .emailId)
        case UserAttribute.gender.rawValue:
            UsageAnalytics.shared.set(value: value!, for: .gender)
        case UserAttribute.language.rawValue:
            UsageAnalytics.shared.set(value: value!, for: .language)
        case UserAttribute.mobileNumber.rawValue:
            UsageAnalytics.shared.set(value: value!, for: .mobileNumber)
        case UserAttribute.name.rawValue:
            UsageAnalytics.shared.set(value: value!, for: .name)
        case UserAttribute.userId.rawValue:
            UsageAnalytics.shared.set(value: value!, for: .userId)
        default:
            UsageAnalytics.shared.set(value: value!, for: key!)
        }
        
        pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "User attribute set Successfully!")
        
        self.commandDelegate!.send(
            pluginResult,
            callbackId: command!.callbackId
        )
    }
    
    @objc(logEvent:)
    func logEvent(_ command: CDVInvokedUrlCommand?) {
        var pluginResult = CDVPluginResult(
            status: CDVCommandStatus_ERROR,
            messageAs: "UserId is required for the SmartKarrot Integration."
        )
        let event:String?
        event = command!.arguments[0] as? String
        let withData = command!.arguments[1] as? [String : Any]
        
        if(event != nil){
            if(withData != nil){
                UsageAnalytics.shared.log(event: event!, with: withData)
            }else{
                UsageAnalytics.shared.log(event: event!)
            }
            
        }
        pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "Event Log Successful")
        
        self.commandDelegate!.send(
            pluginResult,
            callbackId: command!.callbackId
        )
    }
    
    @objc(registerNotificationWithToken:)
    func registerNotificationWithToken(_ command: CDVInvokedUrlCommand?) {
        var pluginResult = CDVPluginResult(
            status: CDVCommandStatus_ERROR,
            messageAs: "UserId is required for the SmartKarrot Integration."
        )
        let dict = command!.arguments![0] as! NSDictionary
        let deviceToken : NSData = NSKeyedArchiver.archivedData(withRootObject: dict) as NSData
        
        
        UsageAnalytics.shared.didRegisterForRemoteNotificationsWithDeviceToken(deviceToken: deviceToken as Data)
        
        pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "Notification Registered")
        
        self.commandDelegate!.send(
            pluginResult,
            callbackId: command!.callbackId
        )
    }
    
    @objc(failRegisterNotificationWithError:)
    func failRegisterNotificationWithError(_ command: CDVInvokedUrlCommand?) {
        var pluginResult = CDVPluginResult(
            status: CDVCommandStatus_ERROR,
            messageAs: "UserId is required for the SmartKarrot Integration."
        )
        let error:Error?
        error = (command!.arguments[0] as! Error)
        
        
        if(error != nil){
            UsageAnalytics.shared.didFailToRegisterForRemoteNotificationsWithError(error: error!)
        }
        pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "Notification registration Failed")
        
        self.commandDelegate!.send(
            pluginResult,
            callbackId: command!.callbackId
        )
    }
    
    @objc(requestUserNotificationsPermission:)
    func requestUserNotificationsPermission(_ command: CDVInvokedUrlCommand?) {
        var pluginResult = CDVPluginResult(
            status: CDVCommandStatus_ERROR,
            messageAs: "Notification Permission Unsuccesful."
        )
        
        UsageAnalytics.shared.requestUserNotificationsPermission()
        
        pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "Notification Permission Succesful")
        
        self.commandDelegate!.send(
            pluginResult,
            callbackId: command!.callbackId
        )
    }
    
    @objc(handleRemoteNotification:)
    func handleRemoteNotification(_ command: CDVInvokedUrlCommand?) {
        var pluginResult = CDVPluginResult(
            status: CDVCommandStatus_ERROR,
            messageAs: "UserId is required for the SmartKarrot Integration."
        )
        let userInfo:Any?
        userInfo = (command!.arguments[0] as! [AnyHashable : Any])
        
        var showDropDown = (command?.arguments[1] as! Bool)
        
        if(userInfo != nil){
            showDropDown = false;
            
            UsageAnalytics.shared.handleRemoteNotification(userInfo: userInfo as! [AnyHashable : Any],showDropdown: showDropDown)
        }
        pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "Notification registration Failed")
        
        self.commandDelegate!.send(
            pluginResult,
            callbackId: command!.callbackId
        )
    }
    
    @objc(handleInAppAnyView:)
    func handleInAppAnyView(_ command: CDVInvokedUrlCommand?) {
        var pluginResult = CDVPluginResult(
            status: CDVCommandStatus_ERROR,
            messageAs: "UserId is required for the SmartKarrot Integration."
        )
        let fromAny:Bool?
        fromAny = (command!.arguments[0] as! Bool)
        //let viewController = (self as CDVPlugin).viewController;
        if(fromAny != nil){
            //UsageAnalytics.shared.handleRemoteNotification(userInfo: userInfo as! [AnyHashable : Any],showDropdown: showDropDown)
            RichPush.shared.handle(fromAny: fromAny ?? false) { (richPushCampaignId, actionName, actionValue) in
                print(richPushCampaignId);
                print(actionName);
                print(actionValue);
            }
        }
        pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "Notification registration Failed")
        
        self.commandDelegate!.send(
            pluginResult,
            callbackId: command!.callbackId
        )
    }
    
    
    
    @objc(configureSurvey:)
    func configureSurvey(_ command: CDVInvokedUrlCommand?) {
        var pluginResult = CDVPluginResult(
            status: CDVCommandStatus_ERROR,
            messageAs: "AppId is required for the SmartKarrot Integration."
        )
        
        Survey.shared.configure();
        pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "SmartKarrot Survey Configuration Successful")
        
        self.commandDelegate!.send(
            pluginResult,
            callbackId: command!.callbackId
        )
    }
    
    @objc(listSurveys:)
    func listSurveys(_ command: CDVInvokedUrlCommand?) {
        var pluginResult = CDVPluginResult(
            status: CDVCommandStatus_ERROR,
            messageAs: "AppId is required for the SmartKarrot Integration."
        )
        
        let isAnimated = command!.arguments[0] as? Bool ?? false;
        let viewController = (self as CDVPlugin).viewController;
        Survey.shared.listSurveys(from: viewController!, animated: isAnimated);
        pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "SmartKarrot Survey List Successful")
        
        self.commandDelegate!.send(
            pluginResult,
            callbackId: command!.callbackId
        )
    }
    
    @objc(startSurvey:)
    func startSurvey(_ command: CDVInvokedUrlCommand?) {
        var pluginResult = CDVPluginResult(
            status: CDVCommandStatus_ERROR,
            messageAs: "Survey Id is required to start the survey."
        )
        let surveyId = command!.arguments[0] as? String ?? nil;
        let viewController = (self as CDVPlugin).viewController;
        let isAnimated = command!.arguments[1] as? Bool ?? false;
        if(surveyId != nil){
            Survey.shared.startSurvey(surveyId: surveyId!, from: viewController!, animated: isAnimated)
            pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "SmartKarrot Survey By Id rendered Successfully!")
        }
        self.commandDelegate!.send(
            pluginResult,
            callbackId: command!.callbackId
        )
    }
    
    @objc(configureReferral:)
    func configureReferral(_ command:CDVInvokedUrlCommand){
        var pluginResult = CDVPluginResult(
            status: CDVCommandStatus_ERROR,
            messageAs: "Referral Configuration UnSuccessful."
        )
       
            Referral.shared.configure()
            pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "SmartKarrot Referral Configured Successfully!")
        
        self.commandDelegate!.send(
            pluginResult,
            callbackId: command.callbackId
        )
    }
    
    @objc(listReferrals:)
    func listReferrals(_ command:CDVInvokedUrlCommand){
        var pluginResult = CDVPluginResult(
            status: CDVCommandStatus_ERROR,
            messageAs: "Referral Configuration UnSuccessful."
        )
         let viewController = (self as CDVPlugin).viewController;
        let isAnimated = command.arguments[0] as? Bool ?? false;
        Referral.shared.listCampaigns(from: viewController!, animated: isAnimated)
        pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "SmartKarrot Referral Configured Successfully!")
        
        self.commandDelegate!.send(
            pluginResult,
            callbackId: command.callbackId
        )
    }
    
    @objc(showDashboard:)
    func showDashboard(_ command:CDVInvokedUrlCommand){
        var pluginResult = CDVPluginResult(
            status: CDVCommandStatus_ERROR,
            messageAs: "Referral Not Found."
        )
        
        let viewController = (self as CDVPlugin).viewController;
        let campaignId = command.arguments[0] as? String;
        let isAnimated = command.arguments[1] as? Bool ?? false;
        
        Referral.shared.showDashboard(campaignId: campaignId!, from: viewController!, animated: isAnimated)
        pluginResult = CDVPluginResult(status: CDVCommandStatus_OK, messageAs: "SmartKarrot Referral Configured Successfully!")
        
        self.commandDelegate!.send(
            pluginResult,
            callbackId: command.callbackId
        )
    }
   
    
    func convertJsonStringToDictionary(text: String) -> [String: Any]? {
        if(text == nil) { return nil }
        if let data = text.replacingOccurrences(of: "\n", with: "").data(using: String.Encoding.utf8) {
            do {
                return try JSONSerialization.jsonObject(with: data, options: []) as? [String:AnyObject]
            } catch {
                print(error.localizedDescription)
            }
        }
        return nil
    }
    
    
    /*----App Delegate Related code ----*/
    @objc(didRegisterForRemoteNotificationsWithDeviceToken:)
    func didRegisterForRemoteNotificationsWithDeviceToken(deviceToken:Data){
        UsageAnalytics.shared.didRegisterForRemoteNotificationsWithDeviceToken(deviceToken: deviceToken as Data)
    }
    
    @objc(didFailToRegisterForRemoteNotificationsWithError:)
    func didFailToRegisterForRemoteNotificationsWithError(error:Error){
        UsageAnalytics.shared.didFailToRegisterForRemoteNotificationsWithError(error: error)
    }
    
    @objc(handleOpenUrl:)
    func handleOpenUrl(url:URL) -> Bool{
        return Referral.shared.handle(url: url)
    }
    
    @objc(handleUniversalLink:)
    func handleUniversalLink(userActivity:NSUserActivity) -> Bool{
        return Referral.shared.handleUniversalLink(userActivity: userActivity)
    }
    
    @objc(didReceiveRemoteNotification:)
    func didReceiveRemoteNotification(userInfo: [AnyHashable : Any]){
        UsageAnalytics.shared.handleRemoteNotification(userInfo: userInfo, showDropdown: true)
    }
    
}



